@@ -10,46 +10,11 @@ class Defog:
         Initializes the Defog class.
         :param api_key: The API key for the defog account.
         """
+        if db_type not in ["postgres", "redshift", "mysql", "bigquery", "mongo"]:
+            raise Exception(f"Database `{db_type}` is not supported right now. db_type must be one of 'postgres', 'redshift', 'mysql', 'bigquery', 'mongo'")
         self.api_key = api_key
         self.db_type = db_type
         self.db_creds = db_creds
-        if db_creds is None:
-            self.update_db_creds()
-    
-    def update_db_creds(self):
-        """
-        Updates the database type.
-        """
-        print("Fetching database credentials...")
-        r = requests.post("https://api.defog.ai/get_db_creds",
-            json={
-                "api_key": self.api_key,
-            }
-        )
-        resp = r.json()
-        db_type = resp['db_type']
-        self.db_type = db_type
-        if db_type == "postgres":
-            creds = resp['postgres_creds']
-            self.db_creds = {
-                "host": creds["postgres_host"],
-                "port": creds["postgres_port"],
-                "database": creds["postgres_db"],
-                "user": creds["postgres_username"],
-                "password": creds["postgres_password"]
-            }
-            return True
-        elif db_type == "mongo":
-            creds = resp['mongo_creds']
-            self.db_creds = {
-                "connection_string": creds["mongo_connection_string"]
-            }
-            return True
-        elif self.db_type == "bigquery":
-            print("Please enter the path to your service account json file when initializing Defog\n\ndefog = Defog(api_key=key, db_type='bigquery', db_creds='path/to/service_account.json)")
-            pass
-        else:
-            raise Exception("Database type not yet supported.")
     
     def generate_postgres_schema(self, tables: list):
         try:
@@ -61,7 +26,7 @@ class Defog:
         cur = conn.cursor()
         schemas = {}
         
-        print("Getting schema for each tables in your database...")
+        print("Getting schema for each table in your database...")
         # get the schema for each table
         for table_name in tables:
             cur.execute("SELECT CAST(column_name AS TEXT), CAST(data_type AS TEXT) FROM information_schema.columns WHERE table_name::text = %s;", (table_name,))
@@ -88,6 +53,43 @@ class Defog:
             print(resp)
             raise resp['message']
     
+    def generate_redshift_schema(self, tables: list):
+        try:
+            import psycopg2
+        except:
+            raise Exception("psycopg2 not installed.")
+        
+        conn = psycopg2.connect(**self.db_creds)
+        cur = conn.cursor()
+        schemas = {}
+        
+        print("Getting schema for each table in your database...")
+        # get the schema for each table
+        for table_name in tables:
+            cur.execute("SELECT CAST(column_name AS TEXT), CAST(data_type AS TEXT) FROM information_schema.columns WHERE table_name::text = %s;", (table_name,))
+            rows = cur.fetchall()
+            rows = [row for row in rows]
+            rows = [{"column_name": i[0], "data_type": i[1]} for i in rows]
+            schemas[table_name] = rows
+        
+        conn.close()
+
+        print("Sending the schema to the defog servers and generating a Google Sheet. This might take up to 2 minutes...")
+        # send the schemas dict to the defog servers
+        r = requests.post("https://api.defog.ai/get_postgres_schema_gsheets",
+            json={
+                "api_key": self.api_key,
+                "schemas": schemas
+            }
+        )
+        resp = r.json()
+        try:
+            gsheet_url = resp['sheet_url']
+            return gsheet_url
+        except Exception as e:
+            print(resp)
+            raise resp['message']
+
     def generate_mysql_schema(self, tables: list):
         try:
             import mysql.connector
@@ -98,7 +100,7 @@ class Defog:
         cur = conn.cursor()
         schemas = {}
         
-        print("Getting schema for each tables in your database...")
+        print("Getting schema for each table in your database...")
         # get the schema for each table
         for table_name in tables:
             cur.execute("SELECT column_name, data_type FROM information_schema.columns WHERE table_name = %s;", (table_name,))
@@ -136,7 +138,7 @@ class Defog:
         
         schemas = {}
         
-        print("Getting schema for each collections in your database...")
+        print("Getting schema for each collection in your database...")
         # get the schema for each table
         for collection_name in collections:
             collection = db[collection_name]
@@ -197,6 +199,20 @@ class Defog:
             print(resp)
             raise resp['message']
     
+    def generate_db_schema(self, tables: list):
+        if self.db_type == "postgres":
+            return self.generate_postgres_schema(tables)
+        elif self.db_type == "mysql":
+            return self.generate_mysql_schema(tables)
+        elif self.db_type == "mongo":
+            return self.generate_mongo_schema(tables)
+        elif self.db_type == "bigquery":
+            return self.generate_bigquery_schema(tables)
+        elif self.db_type == "redshift":
+            return self.generate_redshift_schema(tables)
+        else:
+            raise Exception("Invalid database type. Valid types are: postgres, mysql, mongo, bigquery, and redshift")
+
     def update_mysql_schema(self, gsheet_url : str):
         """
         Updates the postgres schema on the defog servers.
@@ -225,6 +241,20 @@ class Defog:
         resp = r.json()
         return resp
     
+    def update_redshift_schema(self, gsheet_url : str):
+        """
+        Updates the redshift schema on the defog servers.
+        :param gsheet_url: The url of the google sheet containing the schema.
+        """
+        r = requests.post("https://api.defog.ai/update_postgres_schema",
+            json={
+                "api_key": self.api_key,
+                "gsheet_url": gsheet_url
+            }
+        )
+        resp = r.json()
+        return resp
+    
     def update_mongo_schema(self, gsheet_url : str):
         """
         Updates the mongo schema on the defog servers.
@@ -253,6 +283,21 @@ class Defog:
         resp = r.json()
         return resp
     
+    def update_db_schema(self, gsheet_url: str):
+        print("Updating the schema on the Defog servers. This might take a couple minutes...")
+        if self.db_type == "postgres":
+            return self.update_postgres_schema(gsheet_url)
+        elif self.db_type == "mysql":
+            return self.update_mysql_schema(gsheet_url)
+        elif self.db_type == "mongo":
+            return self.update_mongo_schema(gsheet_url)
+        elif self.db_type == "bigquery":
+            return self.update_bigquery_schema(gsheet_url)
+        elif self.db_type == "redshift":
+            return self.update_redshift_schema(gsheet_url)
+        else:
+            raise Exception("Invalid database type. Valid types are: postgres, mysql, mongo, bigquery, and redshift")
+
     def get_query(self, question: str, hard_filters: str = None):
         """
         Sends the query to the defog servers, and return the response.
@@ -289,7 +334,7 @@ class Defog:
         query =  self.get_query(question, hard_filters)
         if query["ran_successfully"]:
             print("Query generated, now running it on your database...")
-            if query['query_db'] == "postgres":
+            if query['query_db'] == "postgres" or query['query_db'] == "redshift":
                 try:
                     import psycopg2
                 except:
